(function (window, App, $) {
    'use strict';
    var ChangeManagement = {
        widget: {
            name: "",
            selectedRowOids: []
        },
        createCAFormRender: function (input) {
            const formOptions = {
                options: input.options.formOptions,
                postRender: input.options.postRender
            };
            
            App.ChangeManagement.widget.name = this.widget.options.id;
            var selectedRows = this.selectedRows();
            App.ChangeManagement.widget.selectedRowOids = [];
            if (selectedRows.length === 0) {
                var contextid = App.page.context.objectIds[0];
                if (!contextid) {
                    App.ChangeManagement.widget.selectedRowOids.push(this.context.objectIds[0]);
                } else {
                    App.ChangeManagement.widget.selectedRowOids.push(App.page.context.objectIds[0]);
                }
            } else {
                for (var i = 0; i < selectedRows.length; i++) {
                    App.ChangeManagement.widget.selectedRowOids.push(selectedRows[i].objectId);
                }
            }
            App.ChangeManagement.createArguments = input.options.createArguments;
            return App.form.createNew(formOptions);
        },
        createCAFormPostRender: function(input) {
            const form = this;
            
            //replace OOTB submit button with ours
            let buttonContainer = form.element.find('.alpaca-form-buttons-container');
            let submitButton = buttonContainer.find('.alpaca-form-button-submit');
            let newSubmitButton = submitButton.clone();
            submitButton.hide();
            newSubmitButton.attr('type', 'button');
            newSubmitButton.text('Submit');
            buttonContainer.append(newSubmitButton);
            newSubmitButton.off('click').on('click', App.ChangeManagement.createCAFormSubmit.bind(form, form));            

            //hide legend
            let legend = form.element.find("legend.alpaca-container-label");
            legend.hide();
        },        
        createCAFormSubmit: function (form) {
            var rowOIDs = "";
            var parameterList = "";
            for (var i = 0 ; i < App.ChangeManagement.widget.selectedRowOids.length; i++) {
                if (rowOIDs != "") {
                    rowOIDs +="&";
                }
                rowOIDs +="emxTableRowId="+App.ChangeManagement.widget.selectedRowOids[i];
            }

            Object.entries(App.ChangeManagement.createArguments).forEach(entry => {
                const key = entry[0];
                const value = entry[1];
                parameterList +="&"+key+"="+value;
            });

            var theForm = this;
            var fields = theForm.getAllFieldInstances();
            var fieldsArray = Object.entries(fields);

            var data = {
                attribute: []
            };
            
            for (var index = 0; index < fieldsArray.length; index++) {
                if (fieldsArray[index][1].options.heliumSettings) {
                    var fieldId = fieldsArray[index][1].options.heliumData.field.keyForData;
                    if ("description" == fieldId) {
                        var descriptionId = fieldsArray[index][1].propertyId;
                        var description = theForm.getFieldInstanceByDataIdentifier(descriptionId).getValue();
                        parameterList = parameterList + "&Description=" + description;
                    }
                    if (fieldsArray[index][1].options.heliumSettings.attributeName) {
                        var attributeName = fieldsArray[index][1].options.heliumSettings.attributeName;
                        var attributeval = "";
                        if (fieldsArray[index][1].options.type == "heliumselect") {
                            attributeval = theForm.getFieldInstanceByDataIdentifier(fieldId).getControlValue();
                        } else if (fieldsArray[index][1].options.type == "pikaday") {
                            var datePicker = theForm.getFieldInstanceByDataIdentifier(fieldId).picker;
                            if (datePicker.getDate() != null) {
                                attributeval = datePicker._d.toLocaleString('en-US',{ year: 'numeric', month: 'long', day: 'numeric' });
                                var millis = datePicker._d.getTime();
                                parameterList +="&"+attributeName+"_msvalue="+millis;
                            }
                        }
                        else {
                            attributeval = theForm.getFieldInstanceByDataIdentifier(fieldId).getValue();
                        }
                        if (attributeval != "") {
                            parameterList +="&"+attributeName+"="+attributeval;
                        }
                    }
                    
                    if (fieldsArray[index][1].options.heliumSettings.relatiosnshipName) {
                        var relationshipName = fieldsArray[index][1].options.heliumSettings.relatiosnshipName;
                        var direction = fieldsArray[index][1].options.heliumSettings.direction;
                        var fieldId = fieldsArray[index][1].options.heliumData.field.keyForData;
                        var connectObjectId = form.getFieldInstanceByDataIdentifier(fieldId).getValue();
                        if (connectObjectId != "") {
                            data.attribute.push({
                                name: fieldId,
                                value: connectObjectId,
                                connect: relationshipName,
                                direction: direction
                            });
                        }
                    }
                }
            }
            
            var params = JSON.stringify({data});
            
            let loader = new App.Loader();
            loader.start();
            App.ajax.call({
                url: App.ajax.getUrl('securityContextService', 'setSecurityContext'),
                dataType: 'json',
                type: 'post',
                async: true
            }).then(function(data) {
                if (data.status == 'Success') {
                    App.ajax.call({
                        url: App.info.actionURL + "InvokeJPOServiceActionWithAttribute?" + rowOIDs + parameterList,
                        type: 'POST',
                        data: params,
                        async: true,
                    }).then(function(message) {
                        loader.stop();
                        theForm.modal.close();
                        if (message.error) {
                            App.toaster.warning(message.error);
                        } else {
                            App.toaster.info("Change Action was created");
                            var dataTable = App.page.getWidgetById(App.ChangeManagement.widget.name).content;
                            dataTable.toggleAll(false); 
                            dataTable.reloadContent();
                        } 
                    }).catch(function (data) {
                        loader.stop();
                        theForm.modal.close();
                        App.toaster.error(data.message);
                    });
                } else if (data.status == 'Warning'){
                    loader.stop();
                    theForm.modal.close();
                    App.toaster.warning(data.message);
                } else {
                    loader.stop();
                    theForm.modal.close();
                    App.toaster.error(data.message);
                }
            }).catch(function (data) {
                loader.stop();
                theForm.modal.close();
                App.toaster.error(data.message);
            });
            return {status:'HANDLED'};
        },
        createChange: function (input) {
            var rowIDs = "";
            var selectedids = this.getSelectedObjectIds();
            var widgetName = this.widget.options.id;
            if (selectedids.length === 0) {
                var contextid = App.page.context.objectIds[0];
                if (!contextid) {
                    rowIDs = "emxTableRowId=" + this.context.objectIds[0] + "&";
                } else {
                    rowIDs = "emxTableRowId=" + App.page.context.objectIds[0] + "&";
                }
            } else {
                for (var i = 0; i < selectedids.length; i++) {
                    rowIDs = rowIDs + "emxTableRowId=" + selectedids[i] + "&";
                }
            }
            App.ajax.call({
                url: App.info.actionURL + "InvokeJPOServiceActionWithAttribute?" + rowIDs + "TypeActual=" + input.options.TypeActual + "&Policy=" + input.options.Policy + "&Vault=" + input.options.Vault + "&program=" + input.options.program + "&function=" + input.options.functionn + "&CreateMode=NewCO&functionality=" + input.options.functionality + "&javaClass=" + input.options.javaClass + "&javaMethod=" + input.options.javaMethod + "&refreshBehavior=refreshRows&json=true&Severity=Low",
                type: 'POST'
            }, false).then(function (message) {
                if (message.error) {
                    App.log.error(message.error);
                } else {
                    App.log.warning("Object created");
                    App.page.getWidgetById(widgetName).refresh();
                }
            })["catch"](function (e) {});
        },
        connectExistingCA: function (input) {
            var rowIDs = "";
            console.log(App.page.context.relIds);
            console.log(App.page.context.relIds.toString());
            var selectedids = App.page.context.relIds;
            if (selectedids.length === 0) {
                rowIDs = "emxTableRowId=" + App.page.context.objectIds[0] + "&";
            } else {
                for (var i = 0; i < selectedids.length; i++) {
                    rowIDs = rowIDs + "emxTableRowId=" + selectedids[i] + "&";
                }
            }
            App.ajax.call({
                url: App.info.actionURL + "InvokeJPOServiceActionWithAttribute?" + rowIDs + "targetObjectId=" + input.dataset.id + "&TypeActual=Change+Action&Policy=Change+Action&Vault=eService+Production&javaClass=com.dassault_systemes.enovia.enterprisechangemgt.common.ChangeAction&javaMethod=connectAffectedItems&refreshBehavior=refreshRows&json=true",
                type: 'POST'
            }, false).then(function (message) {
                if (message.error) {
                    App.toaster.error(message.error);
                } else {
                    App.toaster.info("Change Action was connected");
                    var dataTable = App.page.getWidgetById(App.ChangeManagement.widget.name).content;
                    dataTable.toggleAll(false); 
                    dataTable.reloadContent();
                }
            })["catch"](function (e) {
                App.toaster.error(e.message);
            });
            this.close();
        },
        addExistingCA: function (input) {
            var data = this.selectedRows();
            if (data.length === 0) {
                var contextObj = App.page.context.objectIds;
                if (contextObj.length === 0) {
                    objectIds = this.context.objectIds;
                    App.page.context.relIds = objectIds;
                } else {
                    App.page.context.relIds = contextObj;
                }
            } else {
                var objectIds = [];
                for (var i = 0; i < data.length; i++) {
                    console.log(" rows Selected:" + data[i].objectId);
                    objectIds.push(data[i].objectId);
                }
                App.page.context.relIds = objectIds;
            }
            var widgetName = this.widget.options.id;
            App.ChangeManagement.widget.name = widgetName;
            input.options = {
                "config": "tvc:search:helium:launchpad:common/AddExistingCA.xml"
            };
            App.searchV2.startSearch(input);
        },
        connectExistingCO: function (input) {
            var rowIDs = "";
            console.log(App.page.context.relIds);
            console.log(App.page.context.relIds.toString());
            var selectedids = App.page.context.relIds;
            if (selectedids.length === 0) {
                rowIDs = "emxTableRowId=" + App.page.context.objectIds[0] + "&";
            } else {
                for (var i = 0; i < selectedids.length; i++) {
                    rowIDs = rowIDs + "emxTableRowId=" + selectedids[i] + "&";
                }
            }
            App.ajax.call({
                url: App.info.actionURL + "InvokeJPOServiceActionWithAttribute?" + rowIDs + "targetObjectId=" + input.dataset.id + "&TypeActual=Change+Order&Policy=Change+Order&Vault=eService+Production&javaClass=com.dassault_systemes.enovia.enterprisechangemgt.common.ChangeRequest&javaMethod=connectAffectedItems&refreshBehavior=refreshRows&json=true",
                type: 'POST'
            }, false).then(function (message) {
                if (message.error) {
                    App.toaster.error(message.error);
                } else {
                    App.toaster.info("Change Order was connected");
                    var dataTable = App.page.getWidgetById(App.ChangeManagement.widget.name).content;
                    dataTable.toggleAll(false); 
                    dataTable.reloadContent();
                }
            })["catch"](function (e) {
                App.toaster.error(e.message);
            });

            this.close();
        },
        addExistingCO: function (input) {
            var data = this.selectedRows();
            if (data.length === 0) {
                var contextObj = App.page.context.objectIds;
                if (contextObj.length === 0) {
                    objectIds = this.context.objectIds;
                    App.page.context.relIds = objectIds;
                } else {
                    App.page.context.relIds = contextObj;
                }
            } else {
                var objectIds = [];
                for (var i = 0; i < data.length; i++) {
                    console.log(" rows Selected:" + data[i].objectId);
                    objectIds.push(data[i].objectId);
                }
                App.page.context.relIds = objectIds;
            }
            var widgetName = this.widget.options.id;
            App.ChangeManagement.widget.name = widgetName;
            input.options = {
                "config": "tvc:search:helium:launchpad:common/AddExistingCO.xml"
            };
            App.searchV2.startSearch(input);
        },
        connectExistingCR: function (input) {
            var rowIDs = "";
            console.log(App.page.context.relIds);
            console.log(App.page.context.relIds.toString());
            var selectedids = App.page.context.relIds;
            if (selectedids.length === 0) {
                rowIDs = "emxTableRowId=" + App.page.context.objectIds[0] + "&";
            } else {
                for (var i = 0; i < selectedids.length; i++) {
                    rowIDs = rowIDs + "emxTableRowId=" + selectedids[i] + "&";
                }
            }
            App.ajax.call({
                url: App.info.actionURL + "InvokeJPOServiceActionWithAttribute?" + rowIDs + "targetObjectId=" + input.dataset.id + "&TypeActual=Change+Request&Policy=Change+Request&Vault=eService+Production&javaClass=com.dassault_systemes.enovia.enterprisechangemgt.common.ChangeRequest&javaMethod=connectAffectedItems&refreshBehavior=refreshRows&json=true",
                type: 'POST'
            }, false).then(function (message) {
                if (message.error) {
                    App.toaster.error(message.error);
                } else {
                    App.toaster.info("Change Request was connected");
                    var dataTable = App.page.getWidgetById(App.ChangeManagement.widget.name).content;
                    dataTable.toggleAll(false); 
                    dataTable.reloadContent();
                }
            })["catch"](function (e) {
                App.toaster.error(e.message);
            });
            this.close();
        },
        addExistingCR: function (input) {
            var data = this.selectedRows();
            if (data.length === 0) {
                var contextObj = App.page.context.objectIds;
                if (contextObj.length === 0) {
                    objectIds = this.context.objectIds;
                    App.page.context.relIds = objectIds;
                } else {
                    App.page.context.relIds = contextObj;
                }
            } else {
                var objectIds = [];
                for (var i = 0; i < data.length; i++) {
                    console.log(" rows Selected:" + data[i].objectId);
                    objectIds.push(data[i].objectId);
                }
                App.page.context.relIds = objectIds;
            }
            var widgetName = this.widget.options.id;
            App.ChangeManagement.widget.name = widgetName;
            input.options = {
                "config": "tvc:search:helium:launchpad:common/AddExistingCR.xml"
            };
            App.searchV2.startSearch(input);
        },
        openDashboardInSidePanel2: function (data) {
            var widgetName = this.widget.options.id;
            App.ChangeManagement.widget.name = widgetName;
            if (this.getSelectedObjectIds().length === 0) {
                var selectedRows = App.page.context.objectIds;
            } else {
                var selectedRows = this.getSelectedObjectIds();
                App.page.context.objectIds = selectedRows;
            }
            if (App.page.context.objectIds.length === 0) {
                App.page.context.objectIds = this.context.objectIds;
            }
            console.log(App.page.context.objectIds[0])
            console.log(selectedRows);
            App.page.sidepanel.openDashboard(
                data.options.dashboardRef,
                App.page.context.objectIds[0]);
        },
        createChangePostRender: function () {
            const form = this;
            let buttonContainer = form.element.find('.alpaca-form-buttons-container');
            let submitButton = buttonContainer.find('.alpaca-form-button-submit');
            let newSubmitButton = submitButton.clone();
            submitButton.hide();
            newSubmitButton.attr('type', 'button');
            newSubmitButton.text('Submit');
            buttonContainer.append(newSubmitButton);
            newSubmitButton.off('click').on('click', App.ChangeManagement.SubmitCreateChange.bind(form, form));

            //hide legend
            let legend = form.element.find("legend.alpaca-container-label");
            legend.hide();
        },
        createPartPostRender: function () {
            const form = this;
            let buttonContainer = form.element.find('.alpaca-form-buttons-container');
            let submitButton = buttonContainer.find('.alpaca-form-button-submit');
            let newSubmitButton = submitButton.clone();
            submitButton.hide();
            newSubmitButton.attr('type', 'button');
            newSubmitButton.text('Submit');
            buttonContainer.append(newSubmitButton);
            newSubmitButton.off('click').on('click', App.ChangeManagement.SubmitCreatePart.bind(form, form));
        },
        SubmitCreatePart: async function (form) {
            var data = {
                attribute: []
            };
            var widgetName = App.ChangeManagement.widget.name;
            var fields = form.getAllFieldInstances();
            var fieldsArray = Object.entries(fields);
            var parameterList = "";
            for (var index = 0; index < fieldsArray.length; index++) {
                if (fieldsArray[index][1].options.heliumSettings) {
                    var fieldId = fieldsArray[index][1].options.heliumData.field.keyForData;
                    if (fieldId == "type") {
                        var typeId = fieldsArray[index][1].propertyId;
                        var type = form.getFieldInstanceByDataIdentifier(typeId).getValue();
                        parameterList = parameterList + "&TypeActual=" + type;
                        var program = form.getFieldInstanceByDataIdentifier(typeId).options.heliumSettings.program;
                        var funct = form.getFieldInstanceByDataIdentifier(typeId).options.heliumSettings.function;
                        var postURL = form.getFieldInstanceByDataIdentifier(typeId).options.heliumSettings.postProcessURL;
                        parameterList = parameterList + "&program=" + program + "&function=" + funct;
                        parameterList = parameterList + "&postProcessURL=" + postURL;
                    }
                    if ("AutoName" == fieldsArray[index][1].options.heliumData.field.keyForData) {
                        var key = fieldsArray[index][1].propertyId;
                        if (form.getFieldInstanceByDataIdentifier(key).data.length > 0) {
                            parameterList = parameterList + "&autoNameCheck=true";
                        } else {
                            parameterList = parameterList + "&autoNameCheck=false";
                        }
                    }
                    if ("name" == fieldsArray[index][1].options.heliumData.field.keyForData) {
                        var nameId = fieldsArray[index][1].propertyId;
                        var name = form.getFieldInstanceByDataIdentifier(nameId).getValue();
                        parameterList = parameterList + "&Name=" + name;
                    }					
                    if ("AutoNameSeries" == fieldsArray[index][1].options.heliumData.field.keyForData) {
                        var AutoNameSeriesId = fieldsArray[index][1].propertyId;
                        var AutoNameSeries = form.getFieldInstanceByDataIdentifier(AutoNameSeriesId).getControlValue();
                        parameterList = parameterList + "&AutoNameSeries=" + AutoNameSeries;
                    }
                    if ("ReleaseProcess" == fieldsArray[index][1].options.heliumData.field.keyForData) {
                        var ReleaseProcessId = fieldsArray[index][1].propertyId;
                        var ReleaseProcess = form.getFieldInstanceByDataIdentifier(ReleaseProcessId).getControlValue();
                        parameterList = parameterList + "&ReleaseProcess=" + ReleaseProcess;
                    }
                    if ("DesignCollaboration" == fieldsArray[index][1].options.heliumData.field.keyForData) {
                        var key = fieldsArray[index][1].propertyId;
                        if (form.getFieldInstanceByDataIdentifier(key).data.length > 0) {
                            parameterList = parameterList + "&isVPMVisible=true";
                        } else {
                            parameterList = parameterList + "&isVPMVisible=false";
                        }
                    }
                    if (fieldsArray[index][1].options.heliumSettings.attributeName) {
                        var attributeName = fieldsArray[index][1].options.heliumSettings.attributeName;
                        var attributeval = "";
                        if (fieldsArray[index][1].options.type == "heliumselect") {
                            attributeval = form.getFieldInstanceByDataIdentifier(fieldId).getControlValue();
                        } else if ("checkbox" == fieldsArray[index][1].options.heliumData.field.fieldType) {
                            if (form.getFieldInstanceByDataIdentifier(fieldId).data.length < 0) {
                                attributeval = "TRUE";
                            }
                        } else {
                            attributeval = form.getFieldInstanceByDataIdentifier(fieldId).getValue();
                        }
                        if (attributeval != "") {
                            data.attribute.push({
                                name: attributeName,
                                value: attributeval
                            });
                        }
                        if (fieldsArray[index][1].options.heliumSettings.JpoName) {
                            var jponame = fieldsArray[index][1].options.heliumSettings.JpoName;
                            var method = fieldsArray[index][1].options.heliumSettings.method;
                            data.attribute.push({
                                name: attributeName,
                                value: attributeval,
                                JPOName: jponame,
                                Method: method
                            });
                        }
                    }
                }
            }
            var params = JSON.stringify({
                data
            });
            App.ajax.call({
                url: App.info.actionURL + "InvokeJPOServiceActionWithAttribute?json=true" + parameterList,
                type: 'POST',
                data: params
            }, false).then(function (message) {
                App.page.sidepanel.hide();
                if (message.error) {
                    App.log.error(message.error);
                } else {
                	 App.toaster.info(App.i18n.t('form.createNew.message', {
                         'info': message.objectName,
                         'objectId': message.objectId
                     }), App.i18n.t('form.createNew.title'));
             App.page.getWidgetById(widgetName).refresh();
                }
                if (this.modal) {
                    this.modal.close();
                }
            })["catch"](function (e) {});
        },
        SubmitCreateChange: async function (form) {
            var rowIDs = "";
            var selectedids = App.page.context.objectIds;
            var widgetName = App.ChangeManagement.widget.name;
            if (selectedids.length === 0) {
                rowIDs = "emxTableRowId=" + App.page.context.objectIds[0] + "&";
            } else {
                for (var i = 0; i < selectedids.length; i++) {
                    rowIDs = rowIDs + "emxTableRowId=" + selectedids[i] + "&";
                }
            }
            var data = {
                attribute: []
            };
            var fields = form.getAllFieldInstances();
            var fieldsArray = Object.entries(fields);
            var parameterList = "";
            for (var index = 0; index < fieldsArray.length; index++) {
                if (fieldsArray[index][1].options.heliumSettings) {
                    var fieldId = fieldsArray[index][1].options.heliumData.field.keyForData;
                    if (fieldId == "type") {
                        var typeId = fieldsArray[index][1].propertyId;
                        var type = form.getFieldInstanceByDataIdentifier(typeId).getValue();
                        parameterList = parameterList + "&TypeActual=" + type;
                        var program = form.getFieldInstanceByDataIdentifier(typeId).options.heliumSettings.program;
                        var funct = form.getFieldInstanceByDataIdentifier(typeId).options.heliumSettings.function;
                        var javaClass = form.getFieldInstanceByDataIdentifier(typeId).options.heliumSettings.javaClass;
                        var javaFunction = form.getFieldInstanceByDataIdentifier(typeId).options.heliumSettings.javaMethod;
                        parameterList = parameterList + "&program=" + program + "&function=" + funct;
                        parameterList = parameterList + "&javaClass=" + javaClass + "&javaMethod=" + javaFunction;
                        parameterList = parameterList + "&Vault=" + form.getFieldInstanceByDataIdentifier(typeId).options.heliumSettings.vault;
                    }
                    if ("policy" == fieldsArray[index][1].options.heliumData.field.keyForData) {
                        var policyId = fieldsArray[index][1].propertyId;
                        var Policy = form.getFieldInstanceByDataIdentifier(policyId).getValue();
                        parameterList = parameterList + "&Policy=" + Policy;
                    }
                    if ("description" == fieldsArray[index][1].options.heliumData.field.keyForData) {
                        var descriptionId = fieldsArray[index][1].propertyId;
                        var description = form.getFieldInstanceByDataIdentifier(descriptionId).getValue();
                        parameterList = parameterList + "&Description=" + description;
                    }
                    if ("Severity" == fieldsArray[index][1].options.heliumData.field.keyForData) {
                        var SeverityId = fieldsArray[index][1].propertyId;
                        var Severity = form.getFieldInstanceByDataIdentifier(SeverityId).getControlValue();
                        parameterList = parameterList + "&Severity=" + Severity;
                    }
                    if (fieldsArray[index][1].options.heliumSettings.attributeName) {
                        var attributeName = fieldsArray[index][1].options.heliumSettings.attributeName;
                        var attributeval = "";
                        if (fieldsArray[index][1].options.type == "heliumselect") {
                            attributeval = form.getFieldInstanceByDataIdentifier(fieldId).getControlValue();
                        } else {
                            attributeval = form.getFieldInstanceByDataIdentifier(fieldId).getValue();
                        }
                        if (attributeval != "") {
                            data.attribute.push({
                                name: attributeName,
                                value: attributeval
                            });
                        }
                    }
                    if (fieldsArray[index][1].options.heliumSettings.relatiosnshipName) {
                        var relationshipName = fieldsArray[index][1].options.heliumSettings.relatiosnshipName;
                        var direction = fieldsArray[index][1].options.heliumSettings.direction;
                        var fieldId = fieldsArray[index][1].options.heliumData.field.keyForData;
                        var connectObjectId = form.getFieldInstanceByDataIdentifier(fieldId).getValue();
                        if (connectObjectId != "") {
                            data.attribute.push({
                                name: fieldId,
                                value: connectObjectId,
                                connect: relationshipName,
                                direction: direction
                            });
                        }
                    }
                }
            }
            var params = JSON.stringify({
                data
            });
            let loader = new App.Loader();
            loader.start();
            App.ajax.call({
                url: App.ajax.getUrl('securityContextService', 'setSecurityContext'),
                dataType: 'json',
                type: 'post',
                async: true
            }).then(function(data) {
                if (data.status == 'Success') {
                    App.ajax.call({
                        url: App.info.actionURL + "InvokeJPOServiceActionWithAttribute?" + rowIDs + "CreateMode=NewCO&json=true" + parameterList,
                        type: 'POST',
                        data: params,
                        async: true
                    }, false).then(function(message) {
                        loader.stop();
                        App.page.sidepanel.hide();
                        if (message.error) {
                            App.toaster.warning(message.error);
                            
                        } else {
                            App.toaster.info(form.getFieldInstanceByDataIdentifier(typeId).getValue() + " was created");
                            var dataTable = App.page.getWidgetById(App.ChangeManagement.widget.name).content;
                            dataTable.toggleAll(false); 
                            dataTable.reloadContent();
                        } 
                    }).catch(function (data) {
                        loader.stop();
                        App.page.sidepanel.hide();
                        App.toaster.error(data.message);
                    });
                } else if (data.status == 'Warning'){
                    loader.stop();
                    App.page.sidepanel.hide();
                    App.toaster.warning(data.message);
                } else {
                    loader.stop();
                    App.page.sidepanel.hide();
                    App.toaster.error(data.message);
                }
            }).catch(function (data) {
                loader.stop();
                App.page.sidepanel.hide();
                if (this.modal) {
                    this.modal.close();
                }
                App.toaster.error(data.message);
            });            
        },
        checkDateNotBeforeValueOfField: function (value) {
            if (value === "") {
                return true;
            }
            var startDate = this.options.heliumSettings.dependsOn;
            var EstimatedStartDate = this.parent.childrenByPropertyId[startDate].getValue();
            if (EstimatedStartDate <= value) {
                return true;
            }
            return false;
        },
        checkDateNotInPast: function (value) {
            if (value === "") {
                return true;
            }
            var selectedDate = new Date(value);
            selectedDate.setHours(0, 0, 0, 0);
            var now = new Date();
            now.setHours(0, 0, 0, 0);
            if (now <= selectedDate) {
                return true;
            }
            return false;
        }
    };
    App.ChangeManagement = $.extend({}, App.ChangeManagement || {}, ChangeManagement);
})(window, App, jQuery);
